import argparse
import typing

from kuber import management


def _populate_create(subparsers):
    """..."""
    parser = subparsers.add_parser("create", help="Create the resource bundle.")
    parser.add_argument(
        "--target",
        action="append",
        help=(
            "Specify the resource in the bundle to target with this "
            "operation. Only the resource specified will be acted upon. "
            "Use this flag multiple times in the same call to target "
            "multiple resources. If omitted, the default behavior is "
            "to target all resources in the bundle. Targets are specified "
            "as <NAME> or <KIND>/<NAME> or <NAMESPACE>/<KIND>/<NAME>, "
            "e.g. --target=Namespace/foo, and can contain shell-style "
            "wildcard characters for fuzzy matching."
        ),
    )
    parser.add_argument("--namespace")
    parser.add_argument(
        "--settings",
        action="append",
        help=(
            "Specify a file path or directory to load settings from. "
            "These settings will be available in the ResourceBundle "
            "to use in dynamically configuring the bundle."
        ),
    )


def _populate_status(subparsers):
    """..."""
    parser = subparsers.add_parser("status", help="Resource bundle status.")
    parser.add_argument("--namespace")
    parser.add_argument(
        "--target",
        action="append",
        help=(
            "Specify the resource in the bundle to target with this "
            "operation. Only the resource specified will be acted upon. "
            "Use this flag multiple times in the same call to target "
            "multiple resources. If omitted, the default behavior is "
            "to target all resources in the bundle. Targets are specified "
            "as <NAME> or <KIND>/<NAME> or <NAMESPACE>/<KIND>/<NAME>, "
            "e.g. --target=Namespace/foo, and can contain shell-style "
            "wildcard characters for fuzzy matching."
        ),
    )


def _populate_delete(subparsers):
    """..."""
    parser = subparsers.add_parser("delete", help="Delete the resource bundle.")
    parser.add_argument("--namespace")
    parser.add_argument(
        "--target",
        action="append",
        help=(
            "Specify the resource in the bundle to target with this "
            "operation. Only the resource specified will be acted upon. "
            "Use this flag multiple times in the same call to target "
            "multiple resources. If omitted, the default behavior is "
            "to target all resources in the bundle. Targets are specified "
            "as <NAME> or <KIND>/<NAME> or <NAMESPACE>/<KIND>/<NAME>, "
            "e.g. --target=Namespace/foo, and can contain shell-style "
            "wildcard characters for fuzzy matching."
        ),
    )


def _populate_render(subparsers):
    """..."""
    parser = subparsers.add_parser("render", aliases=["test"], help="Render the bundle")
    parser.add_argument("--namespace")
    parser.add_argument(
        "--settings",
        action="append",
        help=(
            "Specify a file path or directory to load settings from. "
            "These settings will be available in the ResourceBundle "
            "to use in dynamically configuring the bundle."
        ),
    )
    parser.add_argument(
        "--target",
        action="append",
        help=(
            "Specify the resource in the bundle to target with this "
            "operation. Only the resource specified will be acted upon. "
            "Use this flag multiple times in the same call to target "
            "multiple resources. If omitted, the default behavior is "
            "to target all resources in the bundle. Targets are specified "
            "as <NAME> or <KIND>/<NAME> or <NAMESPACE>/<KIND>/<NAME>, "
            "e.g. --target=Namespace/foo, and can contain shell-style "
            "wildcard characters for fuzzy matching."
        ),
    )


def parse_args(
    bundle: "management.ResourceBundle", arguments: typing.Iterable[str] = None
) -> typing.Tuple[argparse.Namespace, typing.List[str]]:
    """Parses the CLI command line arguments."""
    parser = argparse.ArgumentParser(prog=bundle.name)
    subparsers = parser.add_subparsers(
        title="Kuber Resource Bundle Command", dest="command"
    )
    _populate_create(subparsers)
    _populate_delete(subparsers)
    _populate_render(subparsers)
    _populate_status(subparsers)

    if arguments:
        return parser.parse_known_args(args=list(arguments))
    return parser.parse_known_args()



import typing

from kuber.definitions import _abstracts
from kuber import _types


def serialize_property(value: typing.Any) -> typing.Any:
    """
    Converts a Definition property value into one compatible with the json
    and yaml configuration formats. Basic types are returned without any
    conversion, but Definition objects are converted to their dictionary
    representation and lists of non-basic types are converted down as well.

    :param value:
        Property value to be serialized for configuration file format
        compatibility.
    :return:
        Serialized version of the supplied value.
    """
    # Some falsy cases are meaningful and should be preserved.
    nulls: typing.Tuple[None, dict, list] = (None, {}, [])

    if hasattr(value, "to_dict"):
        return value.to_dict() or None

    if isinstance(value, (list, tuple)):
        raw_values = [serialize_property(v) for v in value]
        return [v for v in raw_values if v not in nulls] or None

    if isinstance(value, dict):
        raw_items = {k: serialize_property(v) for k, v in value.items()}
        return {k: v for k, v in raw_items.items() if v not in nulls} or None

    return None if value in nulls else value


def deserialize_property(value: typing.Any, data_type: tuple) -> typing.Any:
    """
    Converts the serialized configuration value into the given kuber data type.
    All Definition objects store data types internally to allow for easily
    mapping from serialized values during the loading of configuration file
    data.

    :param value:
        Serialized configuration value to be converted into the given data
        type.
    :param data_type:
        kuber data type to deserialize the given value into.
    :return:
        Deserialized form of the specified value.
    """
    if value is None:
        return None

    if data_type[0] == _types.integer_or_string:
        return _types.integer_or_string(value)

    if issubclass(data_type[0], _abstracts.Definition):
        return data_type[0]().from_dict(value)

    if data_type[0] == list:
        return [deserialize_property(v, (data_type[1], None)) for v in value]

    return data_type[0](value)


def to_camel_case(source: str) -> str:
    """Converts kebab-case or snake_case to camelCase."""
    parts = source.replace("-", "_").split("_")
    return "".join([parts[0], *[p.capitalize() for p in parts[1:]]])
